# Contributing to UKUI-KWin

## What Needs Doing

There's a large amount of bugs open for UKUI-KWin on our [gitee](https://gitee.com/openkylin/ukui-kwin).

## Where Stuff Is

Everything codewise for UKUI-KWin itself is located in the `src` directory.

### Tab Switcher

The default visual appearance of the tab switcher is not located in the UKUI-KWin repository, and is in fact part of [ukui-window-switch]https://gitee.com/openkylin/ukui-window-switch), located at `windowswitchers `.

### Window Management

Most window management stuff (layouting, movement, properties, communication between client<->server) is defined in files ending with `client`, such as `x11client.cpp` and `xdgshellclient.cpp`.

### Window Effects

Window effects are located in the `effects` folder in `src`, with one folder per effect.
Not everything here is an effect as exposed in the configuration UI, such as the colour picker in `src/effects/colorpicker`.

### Scripting API

Many objects in UKUI-KWin are exposed directly to the scripting API; scriptable properties are marked with Q_PROPERTY and functions that scripts can invoke on them.

Other scripting stuff is located in `src/scripting`.

## Conventions

### Coding Conventions

UKUI-KWin's coding conventions are located [here](doc/coding-conventions.md).

### Commits

We usually use this convention for commits in UKUI-KWin:

```
component/subcomponent: Do a thing

This is a body of the commit message,
elaborating on why we're doing thing.
```

While this isn't a hard rule, it's appreciated for easy scanning of commits by their messages.

## Running UKUI-KWin From Source

UKUI-KWin uses CMake like most KDE projects, so you can build it like this:

```bash
mkdir _build
cd _build
cmake ..
make
```

Running it from your build directory looks like this:
```bash
# from the root of your build directory

cd bin

# for wayland, starts nested session: with console

env QT_PLUGIN_PATH=`pwd` dbus-run-session ./ukui-kwin_wayland --xwayland konsole

# or for x11, replaces current kwin instance:

env QT_PLUGIN_PATH=`pwd` ./kwin_x11 --replace

```

QT_PLUGIN_PATH tells Qt to load UKUI-KWin's plugins from the build directory, and not from your system UKUI-KWin.

The dbus-run-session is needed to prevent the nested UKUI-KWin instance from conflicting with your session UKUI-KWin instance when exporting objects onto the bus, or with stuff like global shortcuts.

## Using A Debugger

Trying to attach a debugger to a running UKUI-KWin instance from within itself will likely be the last thing you do in the session, as UKUI-KWin will freeze until you resume it from your debugger, which you need UKUI-KWin to interact with.

Instead, either attach a debugger to a nested UKUI-KWin instance or debug over SSH.

## Tests

UKUI-KWin has a series of unit tests and integration tests that ensure everything is running as expected.

If you're adding substantial new code, it's expected that you'll write tests for it to ensure that it's working as expected.

If you're fixing a bug, it's appreciated, but not expected, that you add a test case for the bug you fix.

You can read more about [UKUI-KWin's testing infrastructure here](doc/TESTING.md).
